package koreajang;

import java.io.IOException;
import java.io.Reader;
import java.sql.*;

import javax.sql.*;
import javax.naming.*;

import koreajang.bean.ArticleBean;
import koreajang.bean.CommentBean;
import koreajang.etc.KJ;
import koreajang.etc.KJException;
import koreajang.etc.KJList;
import koreajang.etc.KJMap;
import koreajang.etc.KJsqlParamException;


public class KJQuery {
	private final boolean POOL = false;
	private final String DB = "datasource1";
	
	protected Connection conn = null;
	protected PreparedStatement pstmt = null;
	protected ResultSet rs = null;
	

	public static final String nextID = "SELECT LAST_ID FROM ID_SEQUENCE WHERE TABLE_NAME = ?";
	public static final String addID = "UPDATE ID_SEQUENCE SET LAST_ID = LAST_ID+1 WHERE TABLE_NAME = ?";
	public static final String setID = "INSERT INTO ID_SEQUENCE(TABLE_NAME, LAST_ID) VALUES(?, 1)";
			

	
	public int nextID(String query, String tableName) {
		int id = -1;
		connect();
		try {
			pstmt = conn.prepareStatement(query);
			pstmt.setString(1, tableName);
			rs = pstmt.executeQuery();
			
			if(rs.next()) {
				id = rs.getInt(1) + 1;
			} else {
				id = KJ.ArticleStart;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			disconnect();
		}
		
		return id;
	}
			
			
	protected void connect() {
		try {
			if(POOL) {
			/* Ŀؼ Ǯ ̿ */
				Context initContext = new InitialContext();
				DataSource ds = (DataSource)initContext.lookup(DB);
				conn = ds.getConnection();
			} else {
			/* Ŀؼ Ǯ ̿ ʱ */
		        Class.forName("oracle.jdbc.driver.OracleDriver");
				conn = DriverManager.getConnection("JDBC:oracle:thin:@127.0.0.1:1521:XE", "jspbook", "manager");
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	protected void disconnect() {
		if(pstmt != null) {
			try {
				pstmt.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		
		if(conn != null) {
			try {
				conn.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		
		if(rs != null) {
			try {
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		
	}
	
	
	public int updateDB(String query, Object param) {
	// update :  1
		int res = 0;
		connect();
		try {
			res = updateDBunCon(query, param);
		} catch(SQLException e) {
			disconnect();
		}
		return res;
	}
	public int updateDBunCon(String query, Object param) throws SQLException {
	//  :  1
		System.out.println("q : " + query);
		System.out.println("? : " + param);
		pstmt = conn.prepareStatement(query);
		if( param instanceof String) {
			pstmt.setString(1, (String)param);
		} else {
			pstmt.setInt(1, ((Integer)param).intValue());
		}
		return pstmt.executeUpdate();
	}
		
	public int setDB(String query, KJMap params) {
	//  :  n
		int res = 0;
		connect();
		try {
			res = setDBunCon(query, params);
		} catch(Exception e) {
			e.printStackTrace();
		} finally {
			disconnect();
		}
		return res;
	}
	public int setDBunCon(String query, KJMap params) throws Exception {
	//  :  n
		System.out.println("q : " + query.toString());
		pstmt = conn.prepareStatement(query);
		setParams(pstmt, params);
		return pstmt.executeUpdate();
	}

	
	public KJList getList(String query, KJMap params, KJBean bean) {
	//  :  n
		KJList beanList = new KJList();
		
		connect();
		try{
			System.out.println("q : " + query.toString());
			pstmt = conn.prepareStatement(query);
			setParams(pstmt, params);
			
			rs = pstmt.executeQuery();
			while(rs.next()) {
				beanList.add(setKJBeanFromRS(rs, bean));
			}
		}
		catch(Exception e) {
			e.printStackTrace();
			return null;
		}
		finally {
			disconnect();
		}
		
		return beanList;
	}

	public KJBean getDB(String query, KJMap params, KJBean bean) {
	//  :  1
		KJBean kjbean = new KJBean();
		connect();
		try{
			System.out.println("q : " + query.toString());
			pstmt = conn.prepareStatement(query);
			setParams(pstmt, params);
			
			rs = pstmt.executeQuery();
			if(rs.next()) {
				kjbean = (KJBean)setKJBeanFromRS(rs, bean);
			}
			
		}
		catch(Exception e) {
			e.printStackTrace();
			return null;
		}
		finally {
			disconnect();
		}
		
		return kjbean;
	}
	
	public int getInt(String query, Object param) {
		int result = -1;
		connect();
		try {
			pstmt = conn.prepareStatement(query);
			System.out.println("q : " + query.toString());
			if( param instanceof String) {
				pstmt.setString(1, (String)param);
				System.out.println("1S : " + (String)param);
			} else {
				pstmt.setInt(1, ((Integer)param).intValue());
				System.out.println("1I : " + ((Integer)param).intValue());
			}
			rs = pstmt.executeQuery();
			
			if(rs.next()) {
				result = rs.getInt(1);
			} else {
				result = -1;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			disconnect();
		}
		
		return result;
	}
	
	public String getString(String query, Object param) {
		String result = "";
		connect();
		try {
			pstmt = conn.prepareStatement(query);
			System.out.println("q : " + query.toString());
			if( param instanceof String) {
				pstmt.setString(1, (String)param);
				System.out.println("1S : " + (String)param);
			} else {
				pstmt.setInt(1, ((Integer)param).intValue());
				System.out.println("1I : " + ((Integer)param).intValue());
			}
			rs = pstmt.executeQuery();
			
			if(rs.next()) {
				result = rs.getString(1);
			} else {
				result = "";
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			disconnect();
		}
		
		return result;
	}
	

	public String getLongString(String query, KJMap params, String attr) {
	// attr شϴ long string 
		String content = null;
		
		connect();
		Reader reader = null;
		try{
			System.out.println("q : " + query.toString());
			pstmt = conn.prepareStatement(query);
			setParams(pstmt, params);
			
			rs = pstmt.executeQuery();
			if(rs.next()) {
				reader = rs.getCharacterStream(attr);
				System.out.println("c : " + attr);
				char[] buff = new char[512];
				int len = -1;
				StringBuffer buffer = new StringBuffer(512);
				while( (len = reader.read(buff)) != -1 ) {
					buffer.append(buff, 0, len);
				}
				content = buffer.toString();
			}
			
		}
		catch(Exception e) {
			e.printStackTrace();
			return null;
		}
		finally {
			if(reader != null) {
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			disconnect();
		}
		
		return content;
	}
	
	
	public boolean setParams(PreparedStatement pstmt, KJMap params)
	throws KJsqlParamException, SQLException {
		// only String type and Integer type are acceptable.
		for ( int i = 1; params!=null && i <= params.size(); i++ ) {
			if( params.get(i) instanceof String ) {
				pstmt.setString(i, (String)params.get(i));
				System.out.println(i + "S : " + params.get(i));
			} else if( params.get(i) instanceof Integer) {
				pstmt.setInt(i, ((Integer)params.get(i)).intValue());
				System.out.println(i + "I : " + params.get(i));
			} else {
				throw new KJsqlParamException();
			}
		}
		return true;
	}
	
	public KJBean setKJBeanFromRS(ResultSet rs, KJBean bean) throws KJException, SQLException {
		// only ArticleBean(without content) , CommentBean
		if (bean instanceof ArticleBean) {
			//content  ó		
			ArticleBean article = new ArticleBean();
			article.setARTICLE_ID(rs.getInt("article_id"));
			article.setARTICLE_LEVEL(rs.getInt("article_level"));
			article.setCATEGORY(rs.getInt("category"));
			article.setGROUP_ID(rs.getInt("group_id"));
			article.setHIT(rs.getInt("hit"));
			article.setIS_COMMENT(rs.getInt("is_comment"));
			article.setIS_DELETE(rs.getInt("is_delete"));
			article.setORDER_NO(rs.getInt("order_no"));
			article.setPARENT_ID(rs.getInt("parent_id"));
			article.setWRITE_DATE(rs.getTimestamp("write_date"));
			article.setNAME(rs.getString("name"));
			article.setPASSWORD(rs.getString("password"));
			article.setTITLE(rs.getString("title"));
			article.setDATAFILE(rs.getString("datafile"));
			article.setEMAIL(rs.getString("email"));
			return article;
		} else if(bean instanceof CommentBean) {
			CommentBean comment = new CommentBean();
			comment.setARTICLE_ID(rs.getInt("article_id"));
			comment.setCOMMENT_ID(rs.getInt("comment_id"));
			comment.setIS_DELETE(rs.getInt("is_delete"));
			comment.setWRITE_DATE(rs.getTimestamp("write_date"));
			comment.setNAME(rs.getString("name"));
			comment.setPASSWORD(rs.getString("password"));
			comment.setCOMMENTS(rs.getString("comments"));
			return comment;
		}
		else {
			System.out.println("unknowned bean");
			throw new KJsqlParamException();
		}
	}
	

}