package koreajang.dao;

import java.io.StringReader;
import java.sql.SQLException;

import koreajang.KJQuery;
import koreajang.bean.ArticleBean;
import koreajang.etc.KJ;
import koreajang.etc.KJMap;

public class WriteQuery extends KJQuery {

	// get...  default  -1 , 
	// next...  default  1(KJ.Start)	
	public static final String nextGroupID =
		"select max(GROUP_ID) FROM " + KJ.ArticleTable;
	public static final String getGroupID =
		"select GROUP_ID FROM " + KJ.ArticleTable + " WHERE ARTICLE_ID = ?";
	public static final String nextLevel =
		"select ARTICLE_LEVEL FROM " + KJ.ArticleTable + " WHERE ARTICLE_ID = ?";
	public static final String nextOrderNo =
		"select max(ORDER_NO) FROM " + KJ.ArticleTable + " WHERE PARENT_ID = ?" +
			" OR ARTICLE_ID = ?";
	public static final String updateOrderNo =
		"UPDATE " + KJ.ArticleTable + " SET ORDER_NO = ORDER_NO+1" +
			" WHERE GROUP_ID = ? AND ORDER_NO >= ?";
	public static final String writeContent =
		"INSERT INTO " + KJ.ContentTable + " VALUES(?, ?)";
	public static final String writeArticle = 
		"INSERT INTO " + KJ.ArticleTable +
		" ( ARTICLE_ID, PARENT_ID, GROUP_ID, ARTICLE_LEVEL," +
		" ORDER_NO, CATEGORY, IS_COMMENT, IS_DELETE, HIT," +
		" WRITE_DATE, TITLE, NAME, PASSWORD, EMAIL, DATAFILE)" +
		" VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, SYSTIMESTAMP, ?, ?, ?, ?, ?)"; 
	
	
	public void writeArticle(ArticleBean aBean) throws SQLException{
		String query = new String(WriteQuery.writeArticle);
		KJMap params = new KJMap();
		params.put(1, aBean.getARTICLE_ID());
		params.put(2, aBean.getPARENT_ID());
		params.put(3, aBean.getGROUP_ID());
		params.put(4, aBean.getARTICLE_LEVEL());
		
		params.put(5, aBean.getORDER_NO());
		params.put(6, aBean.getCATEGORY());
		params.put(7, aBean.getIS_COMMENT());
		params.put(8, aBean.getIS_DELETE());
		params.put(9, aBean.getHIT());
		
		//params.put(9, aBean.getWRITE_DATE());
		params.put(10, aBean.getTITLE());
		params.put(11, aBean.getNAME());
		params.put(12, aBean.getPASSWORD());
		params.put(13, aBean.getEMAIL());
		params.put(14, aBean.getDATAFILE());

		try {
			connect();
			conn.setAutoCommit(false);
			setDBunCon(query, params);
			writeContent(aBean);
			addID(KJ.ArticleTable, aBean);
			conn.commit();
		} catch(Exception e) {
			e.printStackTrace();
			conn.rollback();
		}
		finally {
			conn.setAutoCommit(true);
			disconnect();
		}
		
	}
	
	private int writeContent(ArticleBean aBean) throws SQLException {
		String query = new String(WriteQuery.writeContent);
		System.out.println("q : " + query);
		System.out.println("1 : " + aBean.getARTICLE_ID());
		System.out.println("2 : " + aBean.getContent());
		
		pstmt = conn.prepareStatement(query);
		pstmt.setString(1, new String(""+aBean.getARTICLE_ID()));
		pstmt.setCharacterStream( 2,
				new StringReader(aBean.getContent()),
				aBean.getContent().length() );
		return pstmt.executeUpdate();
	}
	
	private int addID(String tableName, ArticleBean aBean) throws SQLException  {
		String query = null;
		if( aBean.getARTICLE_ID() == 1 ) {
			query = KJQuery.setID;
		}
		else {
			query = KJQuery.addID;
		}
		return updateDBunCon(query, tableName);
	}
	
	
	
	public int nextLevel(int aid) {
		String query = new String(WriteQuery.nextLevel);
		int level = -1;
		connect();
		try {
			pstmt = conn.prepareStatement(query);
			pstmt.setInt(1, aid);
			
			rs = pstmt.executeQuery();
			
			if(rs.next()){
				level = rs.getInt(1) + 1;
			} else {
				level = KJ.Start;
			}
			
		} catch(Exception e) {
			e.printStackTrace();
			return level;
		}
		finally {
			disconnect();
		}
		
		return level;
	}
	
	public int getGroupID(int aid) {
		String query = new String(WriteQuery.getGroupID);
		int gid = -1;
		connect();
		try {
			pstmt = conn.prepareStatement(query);
			pstmt.setInt(1, aid);
			
			rs = pstmt.executeQuery();
			
			if(rs.next()){
				gid = rs.getInt(1);
			} else {
				gid = -1;
			}
			
		} catch(Exception e) {
			e.printStackTrace();
			return gid;
		}
		finally {
			disconnect();
		}
		
		return gid;
	}
	
	public int nextOrderNo(int gid, int pid) {
		String query = new String(WriteQuery.nextOrderNo);
		String query2 = new String(WriteQuery.updateOrderNo);
		
		int order = -1;
		connect();
		try {
			pstmt = conn.prepareStatement(query);
			pstmt.setInt(1, pid);
			pstmt.setInt(2, pid);
			
			rs = pstmt.executeQuery();
			
			if(rs.next()){
				order = rs.getInt(1) + 1;
				System.out.println("order" + order);
				pstmt = conn.prepareStatement(query2);
				pstmt.setInt(1, gid);
				pstmt.setInt(2, order);
				pstmt.executeUpdate();
			} else {
				order = KJ.Start;
			}
			
		} catch(Exception e) {
			e.printStackTrace();
			return pid;
		}
		finally {
			disconnect();
		}
		
		return order;
	}
	
	public int nextGroupID() {
		String query = new String(WriteQuery.nextGroupID);
		int gid = -1;
		connect();
		try {
			pstmt = conn.prepareStatement(query);
			rs = pstmt.executeQuery();
			
			if(rs.next()){
				gid = rs.getInt(1) + 1;
			} else {
				gid = KJ.Start;
			}
			
		} catch(Exception e) {
			e.printStackTrace();
			return gid;
		}
		finally {
			disconnect();
		}
		
		return gid;
	}
	

	
}
